#!/usr/bin/env perl
#
# Copyright (c) 2009, Microsoft Corporation - All rights reserved.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# LICENSE-GPL in the main directory of this source tree, or the
# BSD license (http://opensource.org/licenses/bsd-license.php).
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Authors:
#   Haiyang Zhang <haiyangz@microsoft.com>
#   Hank Janssen  <hjanssen@microsoft.com>
#

#
# updateinitrd.pl
# This script is used to update the initrd image of the current running system to load
# the vmbus and vsc drivers during booting. By loading these drivers during booting,
# the fstab automount can be used to mount the scsi disk during booting.
#
use strict;
use File::Basename;
use File::Copy;


#
# Declare subroutines.
#

sub get_default_modules_list;
sub backup_initrd_file;

#
# Declare globals.
#

my $kernel_ver = `uname -r`;
my $matchkernel = "vmlinuz-" . $kernel_ver;
my $initrdfile = "";
my @vsc_modules = ();
my $i;

chomp $kernel_ver;
chomp $matchkernel;

#
# main entry point
#
if (@ARGV < 1 or $ARGV[0] eq "--help") {
	print "$0 <list of vsc modules to add to the initrd image >\n";
	exit 1;
}

@vsc_modules = @ARGV;

if (-e "/etc/SuSE-release") {   
	$initrdfile = "/boot/initrd-" . $kernel_ver;	
}
elsif (-e "/etc/redhat-release") {
	$initrdfile = "/boot/initrd-" . $kernel_ver . ".img";	
}
else {
	die "\tUnsupported distro version! You will need to manually update it\n";
}

if (! -e $initrdfile) {
	die "\tUnable to find initrd file ($initrdfile) to update. You will need to manually update it\n";
}
	
backup_initrd_file();

if (-e "/etc/SuSE-release") { 
    my $input_mods = join(' ', @vsc_modules);
	my ($default_mods) = get_default_modules_list();
	chop $default_mods;
	my $new_mods = $default_mods . " " . $input_mods . "\"\n";
	#print "module list - " . $new_mods;

	`mkinitrd -k $matchkernel -i $initrdfile -m $new_mods`;
	
	print "\tDone. Updated $initrdfile\n";
}
elsif (-e "/etc/redhat-release") {
	my $new_mods=" ";
	foreach my $i (0 .. $#vsc_modules) 
	{
		$new_mods = $new_mods . "--with=" . $vsc_modules[$i] . " ";
	}

	#print "module list - " . $new_mods;
	
	`mkinitrd -f $new_mods $initrdfile $kernel_ver`;
	
	print "\tDone. Updated $initrdfile\n";
}
else {
}

#
# backup_initrd_file()
#
sub backup_initrd_file() {
	my $initrdfilebackup = "";
	my $i=0;
	
	$initrdfilebackup = "/boot/" . basename($initrdfile) . ".backup" . $i;	

	while ( -e $initrdfilebackup ) {
		$i ++;
		$initrdfilebackup = "/boot/" . basename($initrdfile) . ".backup" . $i;	
	}
	
	print "\tBacking up $initrdfile to $initrdfilebackup\n";
	copy($initrdfile, "$initrdfilebackup");
}

#
# get_default_modules_list
#
sub get_default_modules_list() {
	my $sysconfig_kern="/etc/sysconfig/kernel";
	my @mods=();
	
	if (!open(SYSCONFIG_KERNEL, "<$sysconfig_kern")) {
		die "Error opening $sysconfig_kern. Running as root?\n";
	}
	
	foreach (<SYSCONFIG_KERNEL>) {
		if ($_ =~ /^INITRD_MODULES/) {
   			#print "Found INITRD_MODULE line - " . $_;
   			chomp $_; 
			@mods = split(/=/,$_);
			last;
		}
	}
	close SYSCONFIG_KERNEL;
	
	return $mods[1];
}
